/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 * 
 * Contributor(s): Nicolas Dolet, Philippe Merle
 */

package org.ow2.frascati.factory.core.instance.binding;

import java.net.URL;
import java.rmi.RMISecurityManager;
import java.util.Map;
import org.objectweb.fractal.bf.connectors.rmi.RmiConnectorConstants;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.types.Policy;
import org.objectweb.fractal.fraclet.types.Step;
import org.eclipse.stp.sca.domainmodel.frascati.FrascatiPackage;
import org.eclipse.stp.sca.domainmodel.frascati.RMIBinding;

/**
 * Bind components using Java RMI Binding.
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version 1.1
 */
@org.objectweb.fractal.fraclet.annotations.Component
public class ScaBindingRMI extends AbstractRemoteBinding <RMIBinding> {

  /**
   * Set java policy when starting RMI plugin.
   */
  @SuppressWarnings("unused")
  @Lifecycle(step=Step.START, policy=Policy.ASYNCHRONOUS)
  private void init() {
    URL policyURL = runtime.getClassLoader().getResource("frascati.policy");
    if (policyURL == null ) {
      log.warning("Java policy file not found for FraSCAti : using default policy");
    } else {
      System.setProperty("java.security.policy", policyURL.toString());
      if (System.getSecurityManager() == null) System.setSecurityManager(new RMISecurityManager());
    }
  }

  // --------------------------------------------------------------------------
  // Implementation of the abstract methods.
  // --------------------------------------------------------------------------

  public String getBindingID() {
    return getID(FrascatiPackage.Literals.RMI_BINDING);
  }

  @Override
  public String getBindingFactoryPluginId() {
    return "rmi";
  }

  @Override
  public void initializeBindingHints(RMIBinding rmiBinding, Map<String, Object> hints) {
    // set protocol specific parameters
    hints.put(RmiConnectorConstants.SERVICE_NAME, rmiBinding.getServiceName());
    hints.put(RmiConnectorConstants.JAVA_RMI_REGISTRY_PORT, rmiBinding.getPort());
    hints.put(RmiConnectorConstants.JAVA_RMI_REGISTRY_HOST_ADDRESS, rmiBinding.getHost());
  }
}
