/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.instance.interfaces;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.fractal.api.type.TypeFactory;

/**
 * Map SCA multiplicity to Fractal Interface Type
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

public class ScaMultiplicity {

  protected static boolean [] ZERO_TO_ONE  = { TypeFactory.OPTIONAL, TypeFactory.SINGLE };
  protected static boolean [] ONE_TO_ONE   = { TypeFactory.MANDATORY, TypeFactory.SINGLE  };
  protected static boolean [] ZERO_TO_MANY = { TypeFactory.OPTIONAL, TypeFactory.COLLECTION };
  protected static boolean [] ONE_TO_MANY  = { TypeFactory.MANDATORY, TypeFactory.COLLECTION };

  protected static Map<String, boolean []> multiplicityType;
  
  static {
    multiplicityType = new HashMap<String, boolean []>();
    multiplicityType.put("0..1", ZERO_TO_ONE);
    multiplicityType.put("1..1", ONE_TO_ONE);
    multiplicityType.put("0..n", ZERO_TO_MANY);
    multiplicityType.put("1..n", ONE_TO_MANY);
  }

  /**
   * Get Fractal contingency value from SCA multiplicity
   * 
   * @param multiplicity - SCA multiplicity constant
   * @return
   */
  
  protected static boolean getcontingency(String multiplicity) {
    return multiplicityType.get(multiplicity)[0];
  }

  /**
   * Get Fractal cardinality value from SCA multiplicity
   * 
   * @param multiplicity - SCA multiplicity constant
   * @return
   */
  
  protected static boolean getcardinality(String multiplicity) {
    return multiplicityType.get(multiplicity)[1];
  }

}
