/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 * 
 * Contributor(s): Nicolas Dolet
 */
package org.ow2.frascati.factory.core.instance.property;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.namespace.QName;

import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.stp.sca.Property;
import org.eclipse.stp.sca.PropertyValue;
import org.eclipse.stp.sca.SCAPropertyBase;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.util.Fractal;
import org.ow2.frascati.factory.core.instance.util.JavaTypeHelper;
import org.ow2.frascati.factory.core.instance.util.XSDTypeHelper;
import org.ow2.frascati.tinfi.control.property.IllegalPromoterException;
import org.ow2.frascati.tinfi.control.property.SCAPropertyController;
import org.apache.commons.lang.StringUtils;
import org.apache.cxf.aegis.type.java5.Java5TypeCreator;
import org.apache.cxf.tools.common.model.JavaType;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.sun.xml.bind.api.impl.NameConverter;

@org.objectweb.fractal.fraclet.annotations.Component
public class ScaPropertyImpl implements ScaProperty {

  /**
   * Logger for this class
   */

  java.util.logging.Logger log = java.util.logging.Logger
      .getLogger(ScaPropertyImpl.class.getCanonicalName());

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.instance.property.ScaProperty#setProperty
   * (org.eclipse .stp.sca.Property, org.objectweb.fractal.api.Component)
   */

  public void setProperty(Property property, Component component)
      throws PropertyException {

    try {

      SCAPropertyController pc = (SCAPropertyController) component
          .getFcInterface("sca-property-controller");

      QName propertyObject = (property.getElement() != null) ? property
          .getElement() : property.getType();

      Node n = getPropertyNode(property);

      Boolean flag = true;

      while (n != null && flag) {

        switch (n.getNodeType()) {

        case Node.ELEMENT_NODE:

          Object o = unmarshal(n, propertyObject);

          // Set property value and class
          pc.setValue(property.getName(), o);
          pc.setType(property.getName(), o.getClass());

          flag = false;

          break;

        case Node.TEXT_NODE:

          log.finer("Resolve simple text property");

          // Property object
          Object prop;

          org.w3c.dom.Text text = (org.w3c.dom.Text) n;

          if (StringUtils.isBlank(text.getTextContent()))
            break;

          // if property type is given in composite file
          if (propertyObject != null) {
            // retrieve type from xsd element
            prop = XSDTypeHelper.getObject(propertyObject, text
                .getTextContent());
          } else {
            // else try to create a new object instance for the corresponding
            // type
            prop = (pc.getDeclaredPropertyType(property.getName()) != null) ? JavaTypeHelper
                .getObject(pc.getDeclaredPropertyType(property.getName()),
                    text.getTextContent())
                : text.getTextContent();
          }

          if ((prop != null))
          // if prop is null implies empty xml node
          {
            // Set property value and class         
            pc.setValue(property.getName(), prop);
            pc.setType(property.getName(), prop.getClass());

            flag = false;
          }
          break;
        }

        n = n.getNextSibling();

      }

    } catch (NoSuchInterfaceException e) {
      throw new PropertyException("Could not get property controller", e
          .getCause());
    } catch (JAXBException e) {
      throw new PropertyException("JAXB fails while unmarshalling property '"
          + property.getName() + "'", e.getCause());
    }

  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.instance.property.ScaProperty#setProperty
   * (org.eclipse .stp.sca.PropertyValue, org.objectweb.fractal.api.Component)
   */

  public void setProperty(PropertyValue property, Component component)
      throws PropertyException {
    try {

      SCAPropertyController pc = (SCAPropertyController) component
          .getFcInterface("sca-property-controller");

      String source = property.getSource();
      if ((source != null) && (!source.equals(""))) {
        if (source.startsWith("$"))
          source = source.substring(1);

        Component superComponents[] = Fractal.getSuperController(component)
            .getFcSuperComponents();

        SCAPropertyController superPc = (SCAPropertyController) superComponents[0]
            .getFcInterface("sca-property-controller");

        superPc.setPromoter(source, pc);

      }

      else {

        QName propertyObject = (property.getElement() != null) ? property
            .getElement() : property.getType();

        Node n = getPropertyNode(property);

        Boolean flag = true;

        while (n != null && flag) {

          switch (n.getNodeType()) {

          case Node.ELEMENT_NODE:

            Object o = unmarshal(n, propertyObject);

            // Set property value and class
            pc.setValue(property.getName(), o);
            pc.setType(property.getName(), o.getClass());

            flag = false;

            break;

          case Node.TEXT_NODE:

            log.finer("Resolve simple text property");

            // Property object
            Object prop;

            org.w3c.dom.Text text = (org.w3c.dom.Text) n;

            if (StringUtils.isBlank(text.getTextContent()))
              break;

            // if property type is given in composite file
            if (propertyObject != null) {
              // retrieve type from xsd element
              prop = XSDTypeHelper.getObject(propertyObject, text
                  .getTextContent());
            } else {
              // else try to create a new object instance for the corresponding
              // type
              prop = (pc.getDeclaredPropertyType(property.getName()) != null) ? JavaTypeHelper
                  .getObject(pc.getDeclaredPropertyType(property.getName()),
                      text.getTextContent())
                  : text.getTextContent();
            }

            if ((prop != null))
            // if prop is null implies empty xml node
            {
              // Set property value and class         
              pc.setValue(property.getName(), prop);
              pc.setType(property.getName(), prop.getClass());

              flag = false;
            }
            break;

          }

          n = n.getNextSibling();

        }
      }

    } catch (NoSuchInterfaceException e) {
      throw new PropertyException("Could not get property controller", e
          .getCause());
    } catch (JAXBException e) {
      throw new PropertyException("JAXB fails while unmarshalling property '"
          + property.getName() + "'", e.getCause());
    } catch (IllegalPromoterException e) {
      throw new PropertyException("Property '" + property.getName()
          + "' cannot be promoted", e.getCause());
    }
  }

  /**
   * Get XML node for an SCA Property
   * 
   * @param property
   *          - SCA property object
   * @return XML Node
   */

  private Node getPropertyNode(SCAPropertyBase property) {

    Map<Object, Object> options = new HashMap<Object, Object>();

    // Set property object as root xml element
    options.put(org.eclipse.emf.ecore.xmi.XMLResource.OPTION_ROOT_OBJECTS,
        Collections.singletonList(property));

    // Save property resource in a new XML document
    Document document = ((XMLResource) property.eResource()).save(null,
        options, null);

    NodeList nList = document.getChildNodes().item(0).getChildNodes();

    // Return XML node defining this SCA Property
    return nList.item(0);

  }

  /**
   * Unmarshal Node into Object using JAXB
   * 
   * @param n
   *          - Property node
   * @param propertyObject
   *          - type retrieve from SCA Property
   * @return unmarshalled object
   * @throws JAXBException
   */

  private Object unmarshal(Node n, QName propertyObject) throws JAXBException {

    // Object unmarshalled with JAXB
    Object obj = null;

    // Retrieve package name
    String packagename = NameConverter.standard.toPackageName(propertyObject
        .getNamespaceURI());

    log.finer("Resolve complex property");
    log.finer("Property package name is : " + packagename);

    JAXBContext jc = JAXBContext.newInstance(packagename);
    Unmarshaller u = jc.createUnmarshaller();

    org.w3c.dom.Element element = (org.w3c.dom.Element) n;

    // unmarshal property node
    obj = u.unmarshal(element);

    return obj;

  }

}
