/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.instance.util;

import org.ow2.frascati.factory.FactoryException;

/**
 * Utility class to create instance of java primitive object from a class and a
 * String value.
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 * @since 1.1
 */

public class JavaTypeHelper {

  enum JavaTypes {
    BYTE, SHORT, INT, LONG, FLOAT, DOUBLE, BOOLEAN, CHAR, STRING
  }

  /**
   * Return Java object instance according to a java class and given string value 
   * 
   * @param cl
   * @param value
   * @return
   * @throws FactoryException
   */
  
  public static Object getObject(Class cl, String value)
      throws FactoryException {

    // retrieve class type name
    String javaClassName = cl.getName().toUpperCase();
    javaClassName = javaClassName.substring(javaClassName.lastIndexOf('.')+1);

    switch (JavaTypes.valueOf(javaClassName)) {
    case BYTE:
      return Byte.parseByte(value);
    case SHORT:
      return Short.parseShort(value);
    case INT:
      return Integer.parseInt(value);
    case LONG:
      return Long.parseLong(value);
    case FLOAT:
      return Float.parseFloat(value);
    case DOUBLE:
      return Double.parseDouble(value);
    case BOOLEAN:
      return Boolean.parseBoolean(value);
    case CHAR:
      return value.charAt(0);
    case STRING:
      return value;
    default:
      // Nothing to do
    }

    throw new FactoryException(
        "Java type is not primitive, Property type need to be defined in the composite definition");

  }

}
