/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.instance.util;

import java.math.BigDecimal;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.namespace.QName;

import org.ow2.frascati.factory.FactoryException;

/**
 * Utility class to convert XSD primitive type into java type
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 * @since 1.1
 */

// TODO to use until an XSD converter will be provided by a third party library
public class XSDTypeHelper {

  private static final String schemaUrl = "http://www.w3.org/2001/XMLSchema";
  private static final String schemaInstanceUrl = "http://www.w3.org/2001/XMLSchema-instance";

  enum XsdTypes {
    STRING, INT, LONG, SHORT, FLOAT, DOUBLE, BOOLEAN, BYTE, UNSIGNEDINT, UNSIGNEDSHORT, UNSIGNEDBYTE, BASE64BINARY, HEXBINARY, TIME, DATE, G, ANYSIMPLETYPE, DURATION, NOTATION, DECIMAL, QNAME, DATETIME
  }

  /**
   * Return Java object instance according to a xsd type and given string value 
   * 
   * @param xsdType
   * @param value
   * @return
   * @throws FactoryException
   */
  
  public static Object getObject(QName xsdType, String value)
      throws FactoryException {

    try {
      DatatypeFactory typeFactory = DatatypeFactory.newInstance();

      String namespace = xsdType.getNamespaceURI();
      String type = xsdType.getLocalPart().toUpperCase();

      if (namespace.equals(schemaUrl) || namespace.equals(schemaInstanceUrl)) {

        switch (XsdTypes.valueOf(type)) {
        case STRING:
          return value;
        case INT :
          return Integer.parseInt(value); 
        case LONG:
          return Long.parseLong(value);
        case SHORT:
          return Short.parseShort(value);
        case FLOAT:
          return Float.parseFloat(value);
        case DOUBLE:
          return Double.parseDouble(value);
        case BOOLEAN:
          return Boolean.parseBoolean(value);
        case BYTE:
          return Byte.parseByte(value);
        case UNSIGNEDINT:
          return Long.parseLong(value);
        case UNSIGNEDSHORT:
          return Integer.parseInt(value);
        case UNSIGNEDBYTE:
          return Short.parseShort(value);
        case BASE64BINARY:
          return value.getBytes();
        case HEXBINARY:
          return value.getBytes();
        case TIME:
          return typeFactory.newXMLGregorianCalendar(value);
        case DATE:
          return typeFactory.newXMLGregorianCalendar(value);
        case G:
          return typeFactory.newXMLGregorianCalendar(value);
        case ANYSIMPLETYPE:
          return value;
        case DURATION:
          return typeFactory.newDuration(value);
        case NOTATION:
          return QName.valueOf(value);
        case DECIMAL:
          return new BigDecimal(value);
        case QNAME:
          return QName.valueOf(value);
        case DATETIME:
          return typeFactory.newXMLGregorianCalendar(value);
        default:
          throw new FactoryException(
              "Error while converting property datatype : unknown XML type");
        }

      }
      throw new FactoryException(
          "Error while converting property datatype : unknown XML type");

    } catch (Exception e) {
      throw new FactoryException("Error while converting property datatype", e);
    }

  }
}
