/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 *
 * Contributor: Philippe Merle
 */

package org.ow2.frascati.factory.core.parser.util;

import org.eclipse.emf.ecore.util.EcoreUtil.Copier;
import org.eclipse.stp.sca.BaseService;
import org.eclipse.stp.sca.Component;
import org.eclipse.stp.sca.ComponentReference;
import org.eclipse.stp.sca.ComponentService;
import org.eclipse.stp.sca.Composite;
import org.eclipse.stp.sca.Interface;
import org.eclipse.stp.sca.Reference;
import org.eclipse.stp.sca.Service;

/**
 * Resolve interfaces for composite references/services and component references.
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

public class CompositeInterfaceResolver implements org.eclipse.stp.sca.introspection.Resolver {

  /**
   * EMF Copier used to copy SCA interfaces.
   */
  private Copier copier = new Copier();

  /**
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.parser.util.Resolver#resolve(org.eclipse.
   * stp.sca.Composite)
   */
  public Composite resolve(Composite composite) {

    // resolve component references
    for (Component component : composite.getComponent()) {
      for (ComponentReference reference : component.getReference()) {
        Interface itf = reference.getInterface();
        BaseService target = reference.getTarget2();
        // check if the reference targets a component service but has no interface
        if ((itf == null) && (target != null)) {
          // Make copy of interface element
          reference.setInterface((Interface) copier.copy(target.getInterface()));
        }
      }
    }

    // resolve composite services
    for (Reference reference : composite.getReference())
      resolveReferenceItf(reference);
    // resolve composite services
    for (Service service : composite.getService())
      resolveServiceItf(service);
    return composite;
  }

  /**
   * Resolve composite reference interface 
   * 
   * @param reference
   */
  public void resolveReferenceItf(Reference reference) {

    Interface itf = reference.getInterface();
    ComponentReference cmpReference = reference.getPromote2();
    
    // check if reference promotes a component but does not have interface
    if ((itf == null) && (cmpReference != null)) {
      // Make copy of interface element
      reference.setInterface((Interface) copier.copy(cmpReference.getInterface()));
    }
  }

  /**
   * Resolve composite service interface 
   * 
   * @param service
   */  
  public void resolveServiceItf(Service service) {

    Interface itf = service.getInterface();
    ComponentService cmpService = service.getPromote2();
    
    // check if service promotes a component but does not have interface
    if ((itf == null) && (cmpService != null)) {
      // Make copy of interface element
      service.setInterface((Interface) copier.copy(cmpService.getInterface()));
    }

  }

}
