/***
 * OW2 FraSCAti Explorer
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Christophe Demarey
 *
 * Contributor(s): Philippe Merle
 * 
 */

package org.ow2.frascati.explorer.action;

import org.ow2.frascati.explorer.ExplorerGUI;

/**
 * Is the FraSCAti Explorer {@link MenuItem} to execute {@link Runnable#run()}
 * in a new {@link Thread}.
 *
 * @author Philippe Merle
 * @author Christophe Demarey
 */
public class RunInNewThreadMenuItem extends AbstractAlwaysEnabledMenuItem<Runnable> {

  /**
   * @see org.ow2.frascati.explorer.action.AbstractAlwaysEnabledMenuItem#execute()
   */
  @Override
  protected void execute(Runnable runnable) {
    runInNewThread(runnable);
  }

  public static void runInNewThread(Runnable runnable) {
    // Creates a new Thread.
    Thread thread = new Thread(runnable);
    // Configures the classloader of the thread.
    thread.setContextClassLoader(ExplorerGUI.getSingleton().getDomainConfig().getClassLoader());
    // Starts the Thread.
    thread.start();
  }
}
