/***
 * OW2 FraSCAti explorer
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Christophe Demarey
 *
 * Contributor(s):
 */
package org.ow2.frascati.explorer.context;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.core.naming.lib.DefaultEntry;


/**
 * Reference context used to specified SCA Bindings as childs of Reference nodes.
 *  
 * @author Christophe Demarey.
 */
public class ServiceContext extends ScaInterfaceContext {

  private static final long serialVersionUID = -6551766956254071130L;

  /**
   * Get entries as defined in the org.objectweb.fractal.explorer.context.InterfaceContext class.
   * 
   * @param object The server interface to wrap.
   * @return entries for interfaces other than client interfaces or control interfaces.
   */
  private List<Entry> getInterfaceContextEntries(Object object) {
    Interface itf = (Interface) object;
    InterfaceType type = (InterfaceType) itf.getFcItfType();
    
    if ( type.isFcClientItf() || type.getFcItfName().endsWith("-controller") )
      return Collections.emptyList();
    
    String signature = type.getFcItfSignature();
    Class[] itfs = object.getClass().getInterfaces();
    List<Entry> entries = new ArrayList<Entry>();
    
    for (Class c : itfs){
        if (signature.equals(c.getName()))
            entries.add( new DefaultEntry(signature, c) );
    }
    return entries;
  }
  
  
  /**
   * @see org.objectweb.util.explorer.api.Context#getEntries(java.lang.Object)
   */
  public Entry[] getEntries(Object object) {
    List<Entry> entries = new ArrayList<Entry>();
    Interface itf = (Interface) object;
    String itfName = itf.getFcItfName();
    Component owner = itf.getFcItfOwner();

    // Add implemented interfaces
    entries.addAll( getInterfaceContextEntries(object) );
    
    // Add entries for SCA bindings
    entries.addAll( getBindingEntries(owner, itf) );
    
    // Add entries for SCA intents
    entries.addAll( getIntentEntries(owner, itfName) );

    return entries.toArray(new Entry[0]);  
  }
}
