/***
 * OW2 FraSCAti fscript
 * Copyright (C) 2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Christophe Demarey
 *
 * Contributor(s):
 */
package org.ow2.frascati.fscript;

import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Map;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.NameController;
import org.objectweb.fractal.util.Fractal;
import org.ow2.frascati.factory.runtime.domain.api.Domain;
import org.ow2.frascati.fscript.model.FraSCAtiModel;

/**
 * This command adds an URL at the class-path used by FraSCAtiScript (especially the
 * embedded SCA Assembly Factory).
 * 
 * @author Christophe Demarey
 */
public class ClassPathAddCommand 
     extends org.objectweb.fractal.fscript.console.ClassPathAddCommand {
  @Override
  public String getName() {
      return "classpath_add";
  }

  @Override
  public String getShortDescription() {
      return "Make a URL available in the classpath.";
  }

  @Override
  public String getLongDescription() {
      return "Syntax: :classpath_add url\n";
  }

  /**
   * Adds an URL at the class-path used by FraSCAtiScript.
   * 
   * @param arg An URL defining a valid path or jar file.
   */
  public void execute(String name) throws Exception {
    URL url = null;
    try {
        url = new URL(name);
    } catch (MalformedURLException e) {
        showError("Invalid URL (" + name + "): " + e.getMessage());
        return;
    }
    Map<String, Object> ctx = getInstanciationContext();
    ClassLoader parent = null;
    if (ctx.containsKey("classloader")) {
        parent = (ClassLoader) ctx.get("classloader");
    } else {
        parent = Thread.currentThread().getContextClassLoader();
    }
    URL[] urls = new URL[] { url };
    ClassLoader cl = new URLClassLoader(urls, parent);
    ctx.put("classloader", cl);
    
    getDomain().addLibraries(urls); // Update FraSCAti classloader
    
    urls = ((URLClassLoader) cl).getURLs();
    for(URL u : urls)
    	showMessage(u.toString());
    showMessage("Classpath updated.");
  }

  /**
   * Get the FraSCAti Domain interface.
   * 
   * @return the FraSCAti domain interface.
   * @throws NoSuchInterfaceException 
   */
  private Domain getDomain() throws NoSuchInterfaceException {
    ContentController cc = Fractal.getContentController(fscript);
    for (Component child : cc.getFcSubComponents()) {
      try {
        NameController nc = Fractal.getNameController(child);
        if ("model".equals(nc.getFcName()))
          return (Domain) Fractal.getBindingController(child).lookupFc(FraSCAtiModel.DOMAIN_ITF);
      } catch (NoSuchInterfaceException nsie) {
        continue;
      }
    }
    return null;
  }
}
