/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.generate.runtime;

import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import java.net.URL;
import java.util.Map;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.Type;
import org.objectweb.fractal.fraclet.annotations.Component;
import org.objectweb.fractal.juliac.Juliac;
import org.objectweb.fractal.juliac.conf.JuliacConfig;
import org.objectweb.fractal.juliac.JuliacException;
import org.objectweb.fractal.juliac.conf.JDKLevel;
import org.objectweb.fractal.util.Fractal;
import org.ow2.frascati.factory.FactoryException;
import org.ow2.frascati.factory.core.instance.runtime.RuntimeFactory;
import org.ow2.frascati.factory.core.instance.runtime.TinfiRuntime;
import org.ow2.frascati.factory.core.instance.InstantiationException;

/**
 * GenericFactory and TypeFactory component service
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

@Component
public class TinfiGenerate extends TinfiRuntime implements RuntimeGenerate,
    RuntimeFactory {

  /**
   * Logger for this class
   */

  java.util.logging.Logger log = java.util.logging.Logger
      .getLogger("assembly-factory");
  
  // Environment variable associated with default output directory
  protected static String FRASCATI_GENERATED = "FRASCATI_GENERATED";

  // Tinfi source code generator
//  protected static String JULIAC_OPT_LEVEL = "org.ow2.frascati.tinfi.opt.oo.FCOOCtrlSourceCodeGenerator:org.objectweb.fractal.juliac.felix.FCOOCtrlSourceCodeGenerator";
  protected static String JULIAC_OPT_LEVEL = "org.ow2.frascati.tinfi.opt.oo.FCOOCtrlSourceCodeGenerator";

  // Output Directory (Juliac BaseDir)
  protected File outputDir;

  // Output directory for generated sources
  protected static String GEN_DIRECTORY = "/target/generated-frascati-sources/";

  // Output directory for compiled classes
  // Must not be loaded by a ClassLoader which is not managed by the Assembly Factory
  protected static String CLASS_DIRECTORY = "/target/generated-frascati-classes/";

  protected Juliac jc;
  protected JuliacConfig jcfg;

  public TinfiGenerate() throws FactoryException{

    super();

    try {

      jc = new Juliac();
      init();

      this.tf = Fractal.getTypeFactory(tinfi);

    } catch (Exception e) {
      throw new FactoryException("Error when initialyzing Tinfi",e);
    } 
  }

  /**
   * (non-Javadoc)
   * 
   * @see org.ow2.frascati.factory.generate.runtime.RuntimeGenerate#addJavaLib(java.lang.String)
   */

  public void addJavaLib(String[] lib) {
    jcfg.addSrclibs(lib);
  }

  /**
   * (non-Javadoc)
   * 
   * @see org.ow2.frascati.factory.generate.runtime.RuntimeGenerate#addJavaSource(java.lang.String)
   */

  public void addJavaSource(String[] source) {

    try {
      jcfg.addSrcs(source);
    } catch (IOException e) {
      throw new FactoryException("Cannot add Java sources",e);
    }

  }

  /**
   * (non-Javadoc)
   * @throws JuliacException 
   * 
   * @see org.ow2.frascati.factory.generate.runtime.RuntimeGenerate#compile()
   */

  public void compile() throws IOException, IllegalArgumentException, JuliacException {
    jc.compile();
    jc.close();
  }

  /**
   * (non-Javadoc)
   * @throws ClassNotFoundException 
   * 
   * @see org.ow2.frascati.factory.generate.runtime.RuntimeGenerate#init(java.io.File)
   */

  public void init() throws IOException, ClassNotFoundException {

    jcfg = new JuliacConfig(jc);
    jcfg.setOptLevel(JULIAC_OPT_LEVEL);
    jcfg.setSourceLevel(JDKLevel.JDK1_5);
    jcfg.setTargetLevel(JDKLevel.JDK1_5);
    jc.setJuliacConfig(jcfg);
    jcfg.loadOptLevels();
    
    Map<String, String> map = System.getenv();
    String defaultOutputDir =  map.get(FRASCATI_GENERATED);
    
    File f = (defaultOutputDir == null) ? new File(".").getAbsoluteFile() : new File(defaultOutputDir);
    
    // Set default output dir
    setOutputDir(f);

  }
  
  /* (non-Javadoc)
   * @see org.ow2.frascati.factory.core.instance.runtime.TinfiRuntime#newInstance(org.objectweb.fractal.api.Type, java.lang.Object, java.lang.Object)
   */

  //Invoke Juliac Compiler, does not return component instance
  
  @Override
  public org.objectweb.fractal.api.Component newInstance(Type arg0,
      Object arg1, Object arg2) throws InstantiationException {
    
      
      try {
        // test if an object array has been given for control desc
        if (arg1.getClass().isArray()) {
          // extract control desc information
          Object[] array = (Object[]) arg1;
          arg1 = array[1];
        }
        jc.getFCSourceCodeGenerator(arg0, arg1, arg2)
            .generate(arg0, arg1, arg2);
        
        return null;
        
      } catch (Exception e) {
        throw new InstantiationException("Cannot generate tinfi component code",e);
      }
      
  }


  /* (non-Javadoc)
   * @see org.ow2.frascati.factory.core.instance.runtime.Loader#setClassLoader(java.lang.ClassLoader)
   */
  
  @Override
  public void setClassLoader(ClassLoader classloader) {
    super.setClassLoader(classloader);
    jcfg.setClassLoader(classloader);
    // this.currentClassloader = classloader;
  }

  public void setOutputDir(File outputDir) {
    if (jcfg == null)
      throw new Error("Must call init() first");

    if (!outputDir.equals(this.outputDir)) {
      try {
        this.outputDir = outputDir;
        jcfg.setBaseDir(outputDir);
        jcfg.setGenDirName(outputDir.getCanonicalPath() + GEN_DIRECTORY);
        jcfg.setClassDirName(outputDir.getCanonicalPath() + CLASS_DIRECTORY);

        // Ensure that target directory is referenced by current classloader
        // May be useless

        URL[] urls = { new File(outputDir.getCanonicalPath() + CLASS_DIRECTORY).toURI().toURL() };
        loadLibraries(urls);
        jcfg.setClassLoader(getClassLoader());
        
//        URLClassLoader cl = new URLClassLoader(urls, getClassLoader());
//        setClassLoader(cl);

      } catch (IOException e) {
        e.printStackTrace();
      }
    }

  }

  /** 
   * Debug output when having exception during component generation/instantiation
   * 
   * @see org.ow2.frascati.factory.core.instance.runtime.Loader#debug(java.lang.ClassLoader)
   */
  
  @Override
  public void debug(ClassLoader cl, PrintStream out) {
    super.debug(cl,out);
    JuliacConfig jcfg = jc.getJuliacConfig();
    out.println("\n -- Juliac configuration details ---");

    String[] list;
    out.println("Base Directory  : " + jcfg.getBaseDir());
    out.println("Root Package    : " + jcfg.getPkgRoot());
    out.println("Directory for generated sources : "
        + jcfg.getGenDirName());
    out.println("Directory for compiled classes  : "
        + jcfg.getClassDirName());

    out.println("\n ---------- Loaded sources");
    list = jcfg.getSrcs();
    for (String src : list) {
      out.println(" " + src);
    }

    list = jcfg.getSrclibs();
    out.println("\n ---------- Loaded libraries");
    for (String lib : list) {
      out.println(" " + lib);
    }

  }

  public File getOutputDir() {
    return jcfg.getBaseDir();
  }

}
