/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 *
 * Contributor: Philippe Merle
 *              Christophe Demarey
 */

package org.ow2.frascati.factory;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URL;

import org.mortbay.log.Log;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.ow2.frascati.factory.core.dispatcher.Manager;
import org.ow2.frascati.factory.core.domain.DomainException;
import org.ow2.frascati.factory.core.instance.util.ScaCompositeHelper;
import org.ow2.frascati.factory.runtime.domain.api.Domain;
import org.ow2.frascati.factory.runtime.domain.api.DomainConfig;

/**
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 */
public class Factory extends AbstractGenerateFactory {
  // Constructors

  public Factory() {
    super();
  }

  // Methods

  /**
   * Load jars and create an SCA assembly instance from composite file
   * 
   * @param composite
   *          - The composite file located into the current class path, on the
   *          local file system, or at an URL
   * @param libs
   *          - Libraries required to load this composite
   * 
   * @return the SCA composite component or null if an error occurs.
   */

  @Override
  public Component getComposite(String composite, URL[] libs) {
    try {

      Domain domain = (Domain) getFactory().getFcInterface("domain");
      return domain.getComposite(composite, libs);

    } catch (NoSuchInterfaceException e) {
      Log.warn("Cannot retrieve the domain interface!");
      e.printStackTrace();
      System.exit(0);
    } catch (DomainException e) {
      e.printStackTrace();
    }
    return null;
  }

  /**
   * Get the name of the ADL definition representing the factory to use.
   */
  @Override
  protected String getFactoryName() {
    return "org.ow2.frascati.factory.FrascatiDomain";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.AbstractGenerateFactory#setOutput(java.lang.String
   * )
   */
  public void setOutput(String output) {
    try {
      DomainConfig config = (DomainConfig) getFactory().getFcInterface(
          "domain-config");
      config.setOutput(new File(output));
    } catch (NoSuchInterfaceException e) {
      Log.warn("Cannot retrieve the domain interface!");
      e.printStackTrace();
      System.exit(0);
    } catch (IOException e) {
      Log.warn("Cannot set output dir to '" + output + "'!");
      e.printStackTrace();
    }
  }

  public Component[] getContribution(String contribution) {

    log.info("Loading contribution file : " + contribution);

    try {
      Domain domain = (Domain) getFactory().getFcInterface("domain");
      Component[] components = domain.getContribution(contribution);
      return components;

    } catch (NoSuchInterfaceException e) {
      log.severe("Cannot retrieve the domain interface!");
      e.printStackTrace();
      System.exit(-1);
    } catch (DomainException e) {
      e.printStackTrace();
    }

    return null;

  }

}
