/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.runtime.domain.api;

import java.net.URL;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotations.Interface;
import org.ow2.frascati.factory.core.domain.DomainException;

/**
 * Interface for domain component
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 * @since 0.6
 */

@Interface(name = "domain-frascati")
public interface Domain {
  
  /** Default service name when exporting this domain using rmi */
  public static final String DEFAULT_DOMAIN_SERVICE_NAME = "frascati";
  
  /** Default port value when exporting this domain using rmi */
  public static final String DEFAULT_DOMAIN_PORT = "1099";
  
  /** Default host name when exporting this domain using rmi */
  public static final String DEFAULT_DOMAIN_HOST = "localhost";
  
  /**
   * Get an SCA Composite instance from composite definition or composite name
   * 
   * @param composite
   *          The composite file located into the current class path, on the
   *          local file system, or at an URL; Or the composite name if it has
   *          already been loaded
   * 
   * @return the SCA composite component or null if an error occurs.
   * @throws DomainException 
   */

  public Component getComposite(String composite) throws DomainException;

  /**
   * Get an SCA Composite instance from composite definition or composite name
   * using given library
   * 
   * @param composite
   *          The composite file located into the current class path, on the
   *          local file system, or at an URL; Or the composite name if it has
   *          already been loaded
   * @param libs
   *          Libraries required to load this composite
   * 
   * @return the SCA composite component or null if an error occurs.
   * @throws DomainException 
   */
  
  public Component getComposite(String composite, URL[] libs) throws DomainException;
  
  /**
   * Load an SCA contribution
   * 
   * @param uri
   * @return
   */
  
  public Component[] getContribution(String contribution) throws DomainException;
  
  /**
   * Get all SCA composites for this domain
   * 
   * @return List of loaded composites
   */
  
  public Component [] getComposites();
  
  /**
   * Remove SCA composite from the domain
   * 
   * @param name - Name of the composite to remove 
   */
  
  public void removeComposite(String name) throws DomainException;
  
  
  /**
   * Add libraries
   * 
   * @return List of loaded composites
   */
  
  public void addLibraries(URL... url);
  
  /**
   * Get all SCA composites names for this domain
   * 
   * @return Composites names list
   */
  
  public String [] getCompositesNames();

}
