/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.runtime.domain.api;

import java.io.File;
import java.io.IOException;
import java.net.URLClassLoader;

import org.objectweb.fractal.fraclet.annotations.Interface;

/**
 * Interface for additional managment of domain component
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 * @since 0.6
 */

@Interface(name = "domain-config")
public interface DomainConfig {
  
  /**
   * Get the ClassLoader used by the Factory of this domain
   * 
   * @return - ClassLoader currently used by the Factory
   */

  public ClassLoader getClassLoader();
  
  /**
   * Set the URLClassLoader used by the Factory of this domain
   * 
   * @param classloader
   *          - URLClassLoader to be used by the Factory
   */

  public void setClassLoader(URLClassLoader classloader);
  
  /**
   * Set output directory for generated files
   * 
   * @param output - output directory
   * @throws IOException 
   */
  
  public abstract void setOutput(File output) throws IOException;
  

}
