/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.Type;
import org.osoa.sca.RequestContext;
import org.ow2.frascati.tinfi.control.content.ContentInstantiationException;
import org.ow2.frascati.tinfi.control.content.SCAContentController;
import org.ow2.frascati.tinfi.osoa.RequestContextImpl;

/**
 * The root class for Tinfi component callback interfaces.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 */
public class TinfiComponentCBInterface<T> extends TinfiComponentInterface<T> {

    public TinfiComponentCBInterface() { super(); }
    public TinfiComponentCBInterface(
        Component owner, String s, Type type, boolean flag, Object obj ) {
        super(owner,s,type,flag,obj);
    }

    
    // -----------------------------------------------------------------------
    // Request context stack and content instance management.
    // These methods are used by generated subclasses.
    // -----------------------------------------------------------------------
    
    /**
     * Push a new request context on the thread-local stack of request contexts
     * and return the content instance corresponding to the current scope
     * policy.
     * 
     * @since 1.1.1
     */
    protected <B> B pushFcAndGet(
        String serviceName, Class<B> businessInterface, B service ) {
        
        // Push a new request context
        SCAContentController cc = getFcSCAContentCtrlItf();
        RequestContext rc =
            new RequestContextImpl<B>(serviceName,businessInterface,service);
        cc.pushRequestContext(rc);        

        // Get a new content instance
        try {
            Object content = cc.getFcContent();
            return (B) content;
        }
        catch( ContentInstantiationException e ) {
            throw new TinfiRuntimeException(e);
        }
    }
    
    /**
     * Release the specified content instance previously retrieved with {@link
     * #getContent()} and pop the request context pushed by {@link
     * TinfiComponentInterceptor#pushFcRequestContext(String, Class, Object)}.
     * 
     * @param content  the content instance which is released
     * @param isEndMethod
     *      <code>true</code> if the method which releases the content instance
     *      is annotated with {@link org.osoa.sca.annotations.EndsConversation}
     * @since 1.1.1
     */
    protected void releaseFcAndPop( Object content, boolean isEndMethod ) {
        
        // Release the content instance
        SCAContentController cc = getFcSCAContentCtrlItf();
        cc.releaseFcContent(content,isEndMethod);
        
        // Pop the request context which has been pushed by pushFcAndGet
        cc.popRequestContext();
    }
    
    
    /**
     * Return the reference of the SCA content controller associated to the
     * current component.
     */
    private SCAContentController getFcSCAContentCtrlItf() {
        try {
            if( cc == null ) {
                cc = (SCAContentController)
                    owner.getFcInterface(SCAContentController.NAME);
            }
            return cc;
          }
          catch( NoSuchInterfaceException e ) {
              throw new TinfiRuntimeException(e);
          }
    }
    private SCAContentController cc;
}
