/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.control.content;

import java.lang.annotation.Annotation;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.ow2.frascati.tinfi.reflect.SetterMethodFilter;
import org.ow2.frascati.tinfi.reflect.Util;

/**
 * This class provides a way for dynamically attaching and retrieving
 * metadata associated with a setter method.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @since 1.1.2
 */
public class InjectionPointMethodImpl<T extends Annotation>
extends InjectionPointImpl<T> {

	private Method setter;
	private Method getter;
	
	/**
     * @throws IllegalArgumentException
     *      if the specified argument is not a setter method
     */
    public InjectionPointMethodImpl( Method setter, T annot )
    throws IllegalArgumentException {
        
    	super(annot);
    	
        SetterMethodFilter.checkSetterMethod(setter);
        this.setter = setter;
        setter.setAccessible(true);  // Enable access to private methods
    }
    
	/**
     * @throws IllegalArgumentException
     *      if the specified argument is not a setter method
     */
    public InjectionPointMethodImpl( Method setter, Method getter, T annot )
    throws IllegalArgumentException {
        
        super(annot);
    	
    	Util.checkMatchingSetterGetterMethods(setter,getter);
        this.setter = setter;
        this.getter = getter;
        setter.setAccessible(true);  // Enable access to private methods
        getter.setAccessible(true);  // Enable access to private methods
    }
    
    public void set( Object target, Object value )
    throws IllegalAccessException, InvocationTargetException {
        setter.invoke(target,value);
    }
    
    public Object get( Object target )
    throws
    	IllegalAccessException, InvocationTargetException,
    	UnsupportedOperationException {
    	
    	if( getter == null ) {
            String msg = "Can not get the value with a setter method";
            throw new UnsupportedOperationException(msg);
    	}
    	
    	Object value = getter.invoke(target);
    	return value;
    }

    /**
     * Return the type of the injection point.
     */
    public Class<?> getType() {
    	Class<?>[] ptypes = setter.getParameterTypes();
    	return ptypes[0];
    }
}
